suppressMessages({
    library(Seurat)
})

# Snakemake variables
seu_dir <- paste0(snakemake@config$result_dir,"/seurat/",snakemake@config$quantifier)
rds <- snakemake@output[[1]] 
isvelo <- snakemake@params$isvelo
seu_proj_name <- snakemake@config$seurat_object$project_name
log_file <- snakemake@log[[1]]
seu_rdata <- snakemake@input$seu_objs   

# log all console output
log <- file(log_file, open="wt")
sink(log)
sink(log, type="message")

# log all variables for debugging purposes
cat('# variables used for this analysis:\n')
cat('seu_dir      <- "', seu_dir,         '"\n', sep = "")
cat('rds          <- "', rds,             '"\n', sep = "")
cat('isvelo       <- "', isvelo,          '"\n', sep = "")
cat('seu_proj_name    <- "', seu_proj_name,          '"\n', sep = "")
cat('seu_rdata    <- c("', paste(seu_rdata, collapse='","'), '")\n', sep = "")
cat('\n')

# Helper function to merge multiple Seurat objects
merge_seu_objects <- function(res){
  cell.ids <- lapply(res,attr, which="project.name")
  names(res) <- unlist(cell.ids)
  if (length(res) > 1) {
    seu_obj <-
      merge(x = res[[1]],
            y = res[2:length(res)],
            add.cell.ids = names(res),
            project = seu_proj_name)
  }  else {
    seu_obj <- res[[1]]
  }
}                             

# Read velocity data if detected (kb with '--workflow Lamanno' argument)
if (isvelo) {
  spliced <- c()
  unspliced <- c()
  for (i in 1:length(seu_rdata)){
    data <- readRDS(seu_rdata[i])
    spliced <- c(spliced,data["sf"])
    unspliced <- c(unspliced, data["uf"])  
  }
  # Merge spliced counts
  seu_obj_sf <- merge_seu_objects(spliced)
  # Merged unspliced counts
  seu_obj_uf <- merge_seu_objects(unspliced)
  # Save workspace
  seu_velo_merged <- c(seu_obj_sf, seu_obj_uf)
  names(seu_velo_merged) <- c("sf_merged","uf_merged")
  saveRDS(seu_velo_merged, file = rds)
} else {
  # Read all RDS object from directory
  res <- sapply(seu_rdata, readRDS)
  # Merge counts
  seu_obj <- merge_seu_objects(res)
  # Save merged object in RDS format
  saveRDS(seu_obj, file = rds)
}
